@@ -88,6 +88,7 @@ from .api_pb2 import (  # type: ignore
     VoiceAssistantResponse,
     VoiceAssistantSetConfiguration,
     VoiceAssistantTimerEventResponse,
+    WaterHeaterCommandRequest,
     ZWaveProxyRequest,
 )
 from .client_base import (
@@ -156,6 +157,8 @@ from .model import (
     VoiceAssistantExternalWakeWord as VoiceAssistantExternalWakeWordModel,
     VoiceAssistantSubscriptionFlag,
     VoiceAssistantTimerEventType,
+    WaterHeaterCommandField,
+    WaterHeaterStateFlag,
     ZWaveProxyRequest as ZWaveProxyRequestModel,
     message_types_to_names,
 )
@@ -163,6 +166,7 @@ from .model_conversions import (
     LIST_ENTITIES_SERVICES_RESPONSE_TYPES,
     SUBSCRIBE_STATES_RESPONSE_TYPES,
 )
+from .object_id import fill_missing_object_ids
 from .util import create_eager_task
 
 _LOGGER = logging.getLogger(__name__)
@@ -171,6 +175,21 @@ DEFAULT_BLE_TIMEOUT = 30.0
 DEFAULT_BLE_DISCONNECT_TIMEOUT = 20.0
 DEFAULT_EXECUTE_SERVICE_TIMEOUT = 30.0
 
+# API version 1.14+ may omit object_id to reduce protocol overhead
+MIN_VERSION_OBJECT_ID_OPTIONAL = APIVersion(1, 14)
+
+
+def _fill_object_ids_if_needed(
+    api_version: APIVersion,
+    entities: list[EntityInfo],
+    device_info: DeviceInfo,
+) -> list[EntityInfo]:
+    """Fill in missing object_id values if API version supports omitting them."""
+    if api_version >= MIN_VERSION_OBJECT_ID_OPTIONAL:
+        return fill_missing_object_ids(entities, device_info)
+    return entities
+
+
 SUBSCRIBE_STATES_MSG_TYPES = (*SUBSCRIBE_STATES_RESPONSE_TYPES, CameraImageResponse)
 
 LIST_ENTITIES_MSG_TYPES = (
@@ -227,6 +246,7 @@ class APIClient(APIClientBase):
     ) -> None:
         # Hook into on_stop handler to clear connection when stopped
         self._connection = None
+        self._cached_device_info = None
         if on_stop:
             self._create_background_task(on_stop(expected_disconnect))
 
@@ -291,11 +311,17 @@ class APIClient(APIClientBase):
         )
         info = DeviceInfo.from_pb(resp)
         self._set_name_from_device(info.name)
+        self._cached_device_info = info
         return info
 
     async def list_entities_services(
         self,
     ) -> tuple[list[EntityInfo], list[UserService]]:
+        # If device_info is not cached, use combined call for efficiency
+        # (sends both requests in a single packet)
+        if (device_info := self._cached_device_info) is None:
+            _, ents, svcs = await self.device_info_and_list_entities()
+            return ents, svcs
         msgs = await self._get_connection().send_messages_await_response_complex(
             (ListEntitiesRequest(),),
             lambda msg: type(msg) is not ListEntitiesDoneResponse,
@@ -313,7 +339,10 @@ class APIClient(APIClientBase):
                 continue
             if cls := response_types[msg_type]:
                 entities.append(cls.from_pb(msg))
-        return entities, services
+        # Fill in missing object_id values using cached device_info
+        api_version = self.api_version
+        assert api_version is not None
+        return _fill_object_ids_if_needed(api_version, entities, device_info), services
 
     async def device_info_and_list_entities(
         self,
@@ -367,7 +396,15 @@ class APIClient(APIClientBase):
                 entities.append(cls.from_pb(msg))
 
         assert device_info is not None
-        return device_info, entities, services
+        self._cached_device_info = device_info
+        # Fill in missing object_id values for entities that don't have them
+        api_version = self.api_version
+        assert api_version is not None
+        return (
+            device_info,
+            _fill_object_ids_if_needed(api_version, entities, device_info),
+            services,
+        )
 
     def subscribe_states(self, on_state: Callable[[EntityState], None]) -> None:
         """Subscribe to state updates."""
@@ -1277,6 +1314,47 @@ class APIClient(APIClientBase):
             req.stop = stop
         self._get_connection().send_message(req)
 
+    def water_heater_command(
+        self,
+        key: int,
+        *,
+        mode: int | None = None,
+        target_temperature: float | None = None,
+        target_temperature_low: float | None = None,
+        target_temperature_high: float | None = None,
+        away: bool | None = None,
+        on: bool | None = None,
+        device_id: int = 0,
+    ) -> None:
+        req = WaterHeaterCommandRequest(key=key, device_id=device_id)
+
+        if mode is not None:
+            req.has_fields |= WaterHeaterCommandField.MODE
+            req.mode = mode
+
+        if target_temperature is not None:
+            req.has_fields |= WaterHeaterCommandField.TARGET_TEMPERATURE
+            req.target_temperature = target_temperature
+
+        if away is not None or on is not None:
+            req.has_fields |= WaterHeaterCommandField.STATE
+            state = WaterHeaterStateFlag(0)
+            if away:
+                state |= WaterHeaterStateFlag.AWAY
+            if on:
+                state |= WaterHeaterStateFlag.ON
+            req.state = state
+
+        if target_temperature_low is not None:
+            req.has_fields |= WaterHeaterCommandField.TARGET_TEMPERATURE_LOW
+            req.target_temperature_low = target_temperature_low
+
+        if target_temperature_high is not None:
+            req.has_fields |= WaterHeaterCommandField.TARGET_TEMPERATURE_HIGH
+            req.target_temperature_high = target_temperature_high
+
+        self._get_connection().send_message(req)
+
     def media_player_command(
         self,
         key: int,
